'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// UPDATED VERSION OF https://github.com/chadsmith/node-liftmaster/blob/master/liftmaster.js
var axios = require('axios');

var constants = require('./constants');

var ErrorHandler = function () {
  function ErrorHandler() {
    _classCallCheck(this, ErrorHandler);
  }

  _createClass(ErrorHandler, null, [{
    key: 'parseBadResponse',
    value: function parseBadResponse(response) {
      if (!response) {
        return ErrorHandler.returnError(12, null, response);
      }

      var data = response.data,
          status = response.status;

      if (!status) {
        return ErrorHandler.returnError(12, null, data);
      }
      if (status === 500) {
        return ErrorHandler.returnError(15);
      }
      if ([400, 401].includes(status)) {
        if (data.code === '401.205') {
          return ErrorHandler.returnError(16, null, data);
        }
        if (data.code === '401.207') {
          return ErrorHandler.returnError(17, null, data);
        }
        return ErrorHandler.returnError(14, null, data);
      }
      if (status === 404) {
        // Return an error for a bad serial number.
        if (data.code === '404.401') {
          return ErrorHandler.returnError(18, null, data);
        }

        // Handle generic 404 errors, likely indicating something wrong with this implementation.
        return ErrorHandler.returnError(20);
      }

      return ErrorHandler.returnError(11, null, data);
    }
  }, {
    key: 'returnError',
    value: function returnError(returnCode, error, response) {
      var result = {
        returnCode: returnCode,
        message: constants.errorMessages[returnCode],
        providerMessage: null,
        unhandledError: null
      };
      if (response && response.description) {
        result.providerMessage = response.description;
      }
      if (error) {
        result.unhandledError = error;
      }
      return Promise.resolve(result);
    }
  }]);

  return ErrorHandler;
}();

var MyQ = function () {
  // Build the object and initialize any properties we're going to use.
  function MyQ(username, password) {
    _classCallCheck(this, MyQ);

    this.accountId = null;
    this.username = username;
    this.password = password;
    this.securityToken = null;
  }

  _createClass(MyQ, [{
    key: 'login',
    value: function login() {
      var _this = this;

      if (!this.username || !this.password) {
        return ErrorHandler.returnError(14);
      }

      return this.executeRequest(constants.routes.login, 'post', null, {
        Username: this.username,
        Password: this.password
      }).then(function (originalResponse) {
        var response = originalResponse.response,
            returnCode = originalResponse.returnCode;

        if (returnCode !== 0) {
          throw originalResponse;
        }
        if (!response || !response.data) {
          return ErrorHandler.returnError(12);
        }

        var data = response.data;

        if (!data.SecurityToken) {
          return ErrorHandler.returnError(11);
        }

        var token = data.SecurityToken;
        _this.securityToken = token;

        return {
          returnCode: 0,
          token: token
        };
      }).catch(function (_ref) {
        var response = _ref.response;
        return ErrorHandler.parseBadResponse(response);
      });
    }
  }, {
    key: 'checkIsLoggedIn',
    value: function checkIsLoggedIn() {
      return !!this.securityToken;
    }
  }, {
    key: 'executeRequest',
    value: function executeRequest(route, method, params, data) {
      var isLoginRequest = route === constants.routes.login;
      var headers = {
        'Content-Type': 'application/json',
        MyQApplicationId: constants.headers.appId,
        'User-Agent': constants.headers.defaultUserAgent,
        ApiVersion: constants.headers.deviceApiVersion,
        BrandId: constants.headers.defaultBrandId,
        Culture: constants.headers.defaultCulture
      };

      // If we aren't logged in or logging in, throw an error.
      if (!isLoginRequest && !this.checkIsLoggedIn()) {
        return ErrorHandler.returnError(13);
      } else if (!isLoginRequest) {
        // Add our security token to the headers.
        headers.SecurityToken = this.securityToken;
      }

      var baseUrl = constants.deviceBase;

      // Authentication routes need a different base URL.
      if ([constants.routes.login, constants.routes.account].includes(route)) {
        baseUrl = constants.authBase;
      }

      var config = {
        method: method,
        baseURL: baseUrl,
        url: route,
        headers: headers,
        data: data,
        params: params
      };

      return axios(config).then(function (response) {
        return {
          returnCode: 0,
          response: response
        };
      });
    }
  }, {
    key: 'getAccountInfo',
    value: function getAccountInfo() {
      var _this2 = this;

      return this.executeRequest(constants.routes.account, 'get', { expand: 'account' }).then(function (returnValue) {
        if (returnValue.returnCode !== 0) {
          return returnValue;
        }
        var data = returnValue.response.data;

        if (!data || !data.Account || !data.Account.Id) {
          return ErrorHandler.returnError(11);
        }
        _this2.accountId = data.Account.Id;

        return null;
      }).catch(function (_ref2) {
        var response = _ref2.response;
        return ErrorHandler.parseBadResponse(response);
      });
    }
  }, {
    key: 'getDevices',
    value: function getDevices() {
      var _this3 = this;

      var promise = Promise.resolve();
      if (!this.accountId) {
        promise = this.getAccountInfo();
      }

      return promise.then(function () {
        return _this3.executeRequest(constants.routes.getDevices.replace('{accountId}', _this3.accountId), 'get');
      }).then(function (returnValue) {
        if (returnValue.returnCode !== 0 && typeof returnValue.returnCode !== 'undefined') {
          return returnValue;
        }

        var data = returnValue.response.data;


        var devices = data.items;
        if (!devices) {
          return ErrorHandler.returnError(11);
        }

        var result = {
          returnCode: 0
        };

        var modifiedDevices = [];
        Object.values(devices).forEach(function (device) {
          var modifiedDevice = {
            family: device.device_family,
            name: device.name,
            type: device.device_type,
            serialNumber: device.serial_number
          };

          var state = device.state;

          if (constants.myQProperties.online in state) {
            modifiedDevice.online = state[constants.myQProperties.online];
          }
          if (constants.myQProperties.doorState in state) {
            modifiedDevice.doorState = state[constants.myQProperties.doorState];
            var date = new Date(state[constants.myQProperties.lastUpdate]);
            modifiedDevice.doorStateUpdated = date.toLocaleString();
          }
          if (constants.myQProperties.lightState in state) {
            modifiedDevice.lightState = state[constants.myQProperties.lightState];
            var _date = new Date(state[constants.myQProperties.lastUpdate]);
            modifiedDevice.lightStateUpdated = _date.toLocaleString();
          }

          modifiedDevices.push(modifiedDevice);
        });

        result.devices = modifiedDevices;
        return result;
      }).catch(function (_ref3) {
        var response = _ref3.response;
        return ErrorHandler.parseBadResponse(response);
      });
    }
  }, {
    key: 'getDeviceState',
    value: function getDeviceState(serialNumber, attributeName) {
      return this.getDevices().then(function (response) {
        var device = (response.devices || []).find(function (d) {
          return d.serialNumber === serialNumber;
        });
        if (!device) {
          return ErrorHandler.returnError(18);
        } else if (!(attributeName in device)) {
          return ErrorHandler.returnError(19);
        }

        var result = {
          returnCode: 0,
          state: device[attributeName]
        };
        return result;
      }).catch(function (_ref4) {
        var response = _ref4.response;
        return ErrorHandler.parseBadResponse(response);
      });
    }
  }, {
    key: 'getDoorState',
    value: function getDoorState(serialNumber) {
      return this.getDeviceState(serialNumber, 'doorState').then(function (result) {
        if (result.returnCode !== 0) {
          return result;
        }

        var newResult = JSON.parse(JSON.stringify(result));
        newResult.doorState = newResult.state;
        delete newResult.state;
        return newResult;
      }).catch(function (_ref5) {
        var response = _ref5.response;
        return ErrorHandler.parseBadResponse(response);
      });
    }
  }, {
    key: 'getLightState',
    value: function getLightState(serialNumber) {
      return this.getDeviceState(serialNumber, 'lightState').then(function (result) {
        if (result.returnCode !== 0) {
          return result;
        }

        var newResult = JSON.parse(JSON.stringify(result));
        newResult.lightState = newResult.state;
        delete newResult.state;
        return newResult;
      }).catch(function (_ref6) {
        var response = _ref6.response;
        return ErrorHandler.parseBadResponse(response);
      });
    }
  }, {
    key: 'setDeviceState',
    value: function setDeviceState(serialNumber, action) {
      var _this4 = this;

      var promise = Promise.resolve();
      if (!this.accountId) {
        promise = this.getAccountInfo();
      }

      return promise.then(function () {
        return _this4.executeRequest(constants.routes.setDevice.replace('{accountId}', _this4.accountId).replace('{serialNumber}', serialNumber), 'put', null, { action_type: action });
      }).then(function (returnValue) {
        var returnCode = returnValue.returnCode;

        if (returnCode !== 0 && typeof returnCode !== 'undefined') {
          return returnValue;
        }

        return {
          returnCode: 0
        };
      }).catch(function (_ref7) {
        var response = _ref7.response;
        return ErrorHandler.parseBadResponse(response);
      });
    }
  }, {
    key: 'setDoorOpen',
    value: function setDoorOpen(serialNumber, shouldOpen) {
      var action = constants.doorCommands.close;

      // Take a precaution and check against the string "false" so
      // that someone doesn't inadvertently open their garage.
      if (shouldOpen && shouldOpen !== 'false') {
        action = constants.doorCommands.open;
      }
      return this.setDeviceState(serialNumber, action);
    }
  }, {
    key: 'setLightState',
    value: function setLightState(serialNumber, turnOn) {
      var action = constants.lightCommands.off;
      if (turnOn) {
        action = constants.lightCommands.on;
      }
      return this.setDeviceState(serialNumber, action);
    }
  }]);

  return MyQ;
}();

module.exports = MyQ;